library braxter_client;

export "src/ibraxter_control.dart";
export "src/annotations.dart";

import "dart:async";
import "dart:html";
import "dart:mirrors";
import "package:braxter_common/braxter_common.dart";
import "src/annotations.dart";
import "src/ibraxter_control.dart";
import "src/trusted_node_validator.dart";

// instance fields

final Map<String, Function> _resolveMap = new Map<String, Function>();
final Map<String, Type> _typeMap = new Map<String, Type>();
final TrustedNodeValidator _validator = new TrustedNodeValidator();

IIoCContainer _container;
String _viewPath;

// properties

List<String> get registeredTags => _resolveMap.keys;
List<Type> get registeredTypes => _typeMap.values;

// public methods

void init(String viewPath, [IIoCContainer container = null]) {
  _viewPath = viewPath;
  _container = container ?? new XerumIocContainer();
}

void run() {
  for (final registeredTag in registeredTags) {
    var tags = querySelectorAll(registeredTag);
    for (var tag in tags) {
      inject(tag.tagName.toLowerCase(), tag);
    }
  }
}

void register(String tag, Type type) {
  document.registerElement(tag, type);
  _container.register(type);
  _typeMap[tag] = type;
  _resolveMap[tag] = () async => _container.resolve(type) as HtmlElement
    ..setInnerHtml(
        await HttpRequest
            .getString("$_viewPath/${tag.replaceAll("-", "_")}.html"),
        validator: new TrustedNodeValidator());
}

Future inject(String elementTag, HtmlElement parent) async {
  var tag = await _resolve(elementTag);
  parent.replaceWith(tag);

  var tagMirror = reflect(tag);
  var tagFields = tagMirror.type.declarations;
  for (VariableMirror variableMirror in tagFields.values) {
    for (var metadata in variableMirror.metadata) {
      if (metadata.hasReflectee && metadata.reflectee.runtimeType == FromView) {
        var selector = (metadata.reflectee as FromView).selector;
        tagMirror.setField(
            tagFields.keys.firstWhere((x) => tagFields[x] == variableMirror),
            querySelector(selector));
      }
    }
  }

  (tag as IBraxterControl)..onInjected();
}

// private method

Future<HtmlElement> _resolve(String tag) async => await _resolveMap[tag]();
